"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.c = exports.o = exports.p = exports.b = void 0;
/**
 * Composes two uniary lambdas together.
 * ```haskell
 * b :: (B -> C) -> (A -> B) -> (A -> C)
 * ```
 * @typeParam B - Refer to signature.
 * @typeParam C - Refer to signature.
 * @typeParam A - Refer to signature.
 */
function b(f) {
    return function (g) {
        return (x) => f(g(x));
    };
}
exports.b = b;
/**
 * Composes a {@link CompositionalStack} from arguments of succeeding calls. The
 * inside-most lambdas are on the left.
 * ```haskell
 * p :: (I -> O1) -> (O1 -> O2) -> (O2 -> O3) -> -- ...
 * ```
 * @typeParam I - Refer to signature.
 * @typeParam O1 - Refer to signature.
 * @typeParam O2 - Refer to signature.
 */
function p(first) {
    function pPartial(next) {
        const composed = b(next)(first);
        return p(composed);
    }
    pPartial.fn = first;
    return pPartial;
}
exports.p = p;
/**
 * Composes a {@link CompositionalStack} from arguments of succeeding calls. The
 * inside-most lambdas are on the right.
 * ```haskell
 * o :: (I1 -> O) -> (I2 -> I1) -> (I3 -> I2) -> -- ...
 * ```
 * @typeParam O - Refer to signature.
 * @typeParam I1 - Refer to signature.
 * @typeParam I2 - Refer to signature.
 */
function o(outside) {
    function oPartial(inside) {
        const composed = b(outside)(inside);
        return o(composed);
    }
    oPartial.fn = outside;
    return oPartial;
}
exports.o = o;
/**
 * Builds a {@link CompositionalStack} into a callable function.
 * ```haskell
 * c :: CompositionalStack I O -> I -> O
 * ```
 * @typeParam I - Refer to signature.
 * @typeParam O - Refer to signature.
 */
function c(stack) {
    return stack.fn;
}
exports.c = c;
