/**
 * Generic uniary lambda.
 * ```haskell
 * Lmbd :: A -> B
 * ```
 * @typeParam A - The argument.
 * @typeParam B - The result.
 */
interface Lmbd<A, B> {
    (x: A): B;
}
/**
 * A functional composition stack. The result of either `p` or `o`. The built
 * function will have the signature.
 * ```haskell
 * f :: I -> O
 * ```
 * @see p
 * @see o
 * @typeParam I - Refer to signature.
 * @typeParam O - Refer to signature.
 */
interface CompositionalStack<I, O> extends Function {
    fn: Lmbd<I, O>;
}
/**
 * Composes two uniary lambdas together.
 * ```haskell
 * b :: (B -> C) -> (A -> B) -> (A -> C)
 * ```
 * @typeParam B - Refer to signature.
 * @typeParam C - Refer to signature.
 * @typeParam A - Refer to signature.
 */
export declare function b<B, C>(f: Lmbd<B, C>): <A>(g: Lmbd<A, B>) => (x: A) => C;
/**
 * Composes a {@link CompositionalStack} from arguments of succeeding calls. The
 * inside-most lambdas are on the left.
 * ```haskell
 * p :: (I -> O1) -> (O1 -> O2) -> (O2 -> O3) -> -- ...
 * ```
 * @typeParam I - Refer to signature.
 * @typeParam O1 - Refer to signature.
 * @typeParam O2 - Refer to signature.
 */
export declare function p<O1, I>(first: Lmbd<I, O1>): {
    <O2>(next: Lmbd<O1, O2>): {
        <O2_1>(next: Lmbd<O2, O2_1>): {
            <O2_2>(next: Lmbd<O2_1, O2_2>): {
                <O2_3>(next: Lmbd<O2_2, O2_3>): {
                    <O2_4>(next: Lmbd<O2_3, O2_4>): {
                        <O2_5>(next: Lmbd<O2_4, O2_5>): {
                            <O2_6>(next: Lmbd<O2_5, O2_6>): {
                                <O2_7>(next: Lmbd<O2_6, O2_7>): {
                                    <O2_8>(next: Lmbd<O2_7, O2_8>): {
                                        <O2_9>(next: Lmbd<O2_8, O2_9>): {
                                            <O2_10>(next: Lmbd<O2_9, O2_10>): any;
                                            fn: Lmbd<I, O2_9>;
                                        };
                                        fn: Lmbd<I, O2_8>;
                                    };
                                    fn: Lmbd<I, O2_7>;
                                };
                                fn: Lmbd<I, O2_6>;
                            };
                            fn: Lmbd<I, O2_5>;
                        };
                        fn: Lmbd<I, O2_4>;
                    };
                    fn: Lmbd<I, O2_3>;
                };
                fn: Lmbd<I, O2_2>;
            };
            fn: Lmbd<I, O2_1>;
        };
        fn: Lmbd<I, O2>;
    };
    fn: Lmbd<I, O1>;
};
/**
 * Composes a {@link CompositionalStack} from arguments of succeeding calls. The
 * inside-most lambdas are on the right.
 * ```haskell
 * o :: (I1 -> O) -> (I2 -> I1) -> (I3 -> I2) -> -- ...
 * ```
 * @typeParam O - Refer to signature.
 * @typeParam I1 - Refer to signature.
 * @typeParam I2 - Refer to signature.
 */
export declare function o<I1, O>(outside: Lmbd<O, I1>): {
    <I2>(inside: Lmbd<I2, O>): {
        <I2_1>(inside: Lmbd<I2_1, I2>): {
            <I2_2>(inside: Lmbd<I2_2, I2_1>): {
                <I2_3>(inside: Lmbd<I2_3, I2_2>): {
                    <I2_4>(inside: Lmbd<I2_4, I2_3>): {
                        <I2_5>(inside: Lmbd<I2_5, I2_4>): {
                            <I2_6>(inside: Lmbd<I2_6, I2_5>): {
                                <I2_7>(inside: Lmbd<I2_7, I2_6>): {
                                    <I2_8>(inside: Lmbd<I2_8, I2_7>): {
                                        <I2_9>(inside: Lmbd<I2_9, I2_8>): {
                                            <I2_10>(inside: Lmbd<I2_10, I2_9>): any;
                                            fn: Lmbd<I2_9, I1>;
                                        };
                                        fn: Lmbd<I2_8, I1>;
                                    };
                                    fn: Lmbd<I2_7, I1>;
                                };
                                fn: Lmbd<I2_6, I1>;
                            };
                            fn: Lmbd<I2_5, I1>;
                        };
                        fn: Lmbd<I2_4, I1>;
                    };
                    fn: Lmbd<I2_3, I1>;
                };
                fn: Lmbd<I2_2, I1>;
            };
            fn: Lmbd<I2_1, I1>;
        };
        fn: Lmbd<I2, I1>;
    };
    fn: Lmbd<O, I1>;
};
/**
 * Builds a {@link CompositionalStack} into a callable function.
 * ```haskell
 * c :: CompositionalStack I O -> I -> O
 * ```
 * @typeParam I - Refer to signature.
 * @typeParam O - Refer to signature.
 */
export declare function c<I, O>(stack: CompositionalStack<I, O>): Lmbd<I, O>;
export {};
